# This source code is part of the Biotite package and is distributed
# under the 3-Clause BSD License. Please see 'LICENSE.rst' for further
# information.

"""
Transfer structure data from Biotite to PyMOL.
"""

__version__ = "0.1.1"
__name__ = "biotite2pymol"
__author__ = "Patrick Kunzmann"

from .display import *
from .model import *
from .selection import *
from .startup import *